//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

/*

	Classname:  rmXPTabBook

   Version:    1.05.0723
    
   Date:       May 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmXPTabBookFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

   Disclaimer: It is the duty of the user to determine it's 
               suitability for use. In no circumstance will the
               author accept any responsibility for any 
               consequence arising from it's use.
	
   
   Description:

      A custom XP style "fake" tabbed notebook which uses and switches form
      pages in order to display its controls. This control can be used to 
      provide an XP look notebook type control to Win95/98/ME NT or Win2000.

      If run under XP, it will use the XP colour scheme in force, but this can
      be explicitly set by the developer if required.

      Since the background for this control is a gradient bitmap, text controls
      need to have their background colour matched to their vertical position 
      in the rmXPTabBook control. Custom TextLabel and Text controls are
      appended which will automatically find their associated rmXPTabBook
      control, and set their background colours accordingly to match the bitmap
      or defined background colour of the appropriate rmXPTabBook page. This
      does however create an overhead, and if this is a problem when using a
      large number of text controls, or with old slow machines, it may be better
      to hard code the background colours accordingly.


   Revisions:

      2005/07/23

         Minor bug fix (typo) - The control now opens with the correct colour
         under XP when HomeStead/Olive Green is the active XP colour.


   Methods:

      CreateTab(cTabText, cTabColour, nTabTargetPage, nTabWidth, cTabIconName)

         The onOpen method of the control should be used to create and define
         the required tabs as follows : 

            function RMXPTABBOOK1_onOpen()
               // Create tabs
               this.CreateTab("First Tab", "0xFFFFFF", 1, 59)
               this.CreateTab("Very Wide Tab", "0xFFFFFF", 2, 101, "ICON_DESIGN")
               this.CreateTab("Tab", "0xFFFFFF", 3, 41, "ICON_REDARROWRIGHT")
               this.CreateTab("", "0xFFFFFF", 4, 26, "TS_CUT")
               this.CreateTab("Last Tab", "0xD9FFFF", 5, 67, "ICON_KEY")
               // Select initial tab
               this.PaintTab1.onLeftMouseDown()
            return

         Note that a hex tab colour of "0xFFFFFF" indicates that the XP gradient
         bitmap background should be used for the TabBook page area, any other
         hex colour will be rendered as such.

         cTabIconName can be any bitmap icon resource found in rmXPLook.dll.

         All the TS_ resources from the dBASE resource.dll are available in
         rmXPLook.dll. Feel free to add more bitmaps to this dll if you wish,
         and send a copy back to me. So long as they meet a basic quality
         standard they will be included in future releases.

         The ideal icon height is 14 pixels.

         The transparent colour should be hex "800080" or RGB 128,0,128
         which is the standard dBASE transparent colour.


      SetXPColour(cColour)

         This method is used to specifically define the XP colour scheme to use,
         and accepts a parameter from the following :

            "Default (blue)" "Blue"

            "Metallic" "Silver"

            "Homestead" "Olive Green"


   Useage:

      See rmXPTabBook.wfm for sample code.


   Dependancies:

      rmPaintbox.cc - Base paintbox class

      rmPaintbox.h  - Header file

      rmXPLook.dll  - Bitmap resource
      

   Acknowledgements:

      None.  
         
*/

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

#include rmPaintBox.h

Class rmXPTabBook(oF) of Container(oF) Custom

   set procedure to rmPaintbox.cc additive

   with (this)
      parent.metric := 6 // Pixels 
      transparent = true
      borderStyle = 3	// None
      width = 200
      height = 100
      pageno = 0
   endwith

   // Custom Properties
   this.TabCount = 0
   this.NextTabLeft = 0
   this.CurrentColour = "C0C0C0"
   this.CurrentTabLeft = 0
   this.CurrentTabWidth = 0
   this.XPColour = "BLUE"       // "HOMESTEAD" "METALLIC"
   this.Disabled = false
   this.aTabRefs = new array()

   // Check if running on XP or not
   this.IsXP = false
   this.ThemeName = ""
   this.ColorName = ""
   this.SizeName = ""
   try
      load dll uxtheme.dll
      release dll uxtheme.dll
      this.IsXP := true
      if type("GetCurrentThemeName") # "FP" 
          extern CUINT GetCurrentThemeName(CPTR,CUINT,CPTR,CUINT,CPTR,CUINT) uxtheme 
      endif
   catch( exception e )
      this.IsXP := false
   endtry

   // Get XP ColorName in use
   if this.IsXP = true
      local cThemeFileName, cColorName, cSizeName  
      cThemeFileName = space(260)
      cColorName = space(260)
      cSizeName = space(260)
      this.ThemeName = ""
      this.ColorName = ""
      this.SizeName = ""
      nRet = GetCurrentThemeName(cThemeFileName, 260, cColorName, 260, cSizeName, 260)
      if nRet == 0
         this.ThemeName := trim(cThemeFileName)
         this.ColorName := trim(cColorName)
         this.SizeName  := trim(cSizeName)
      endif
   endif
   do case
      case this.ColorName == "Metallic"    // "Silver"
         this.XPColour = "METALLIC"
      case this.ColorName == "HomeStead"   // "Olive Green"
         this.XPColour = "HOMESTEAD"
      otherwise // case this.ColorName == "NormalColor" // "Default (blue)"
         this.XPColour = "BLUE"
   endcase
   
   // -------------------------------------

   this.PageArea = new rmPaintBox(this)
   with (this.PageArea)
      onPaint = class::PageAreaOnPaint
      left = 0
      top = 20
      width = 200
      height = 80
      ColorNormal =   "N/" + this.parent.ColorNormal
   endwith

   // Load Bitmap Resource dll
   this.PageArea.LoadDLL("rmXPLook.dll")

   // -------------------------------------

   function SetXPColour(cColour)

      // Set colour property
      do case
         case cColour == "Metallic" or cColour == "Silver"
            this.XPColour := "METALLIC"
         case cColour == "Homestead" or cColour == "Olive Green"
            this.XPColour := "HOMESTEAD"
         otherwise
            this.XPColour := "BLUE"
      endcase

      // Repaint control
      local nCount
      for nCount = 1 to this.aTabRefs.size
         if this.aTabRefs[nCount].Selected == true
            this.aTabRefs[nCount].Selected := false
            this.aTabRefs[nCount].onLeftMouseDown()
         endif
      endfor // next 

   return

   // -------------------------------------

   function PageAreaOnPaint()

      local hDC, hDefPen, hDefBrush, xPaint, tw, th 

      this.width = (this.parent.width)           
      this.height = (this.parent.height - 20)   
      tw = this.width
      th = this.height
       
      xPaint = space(32) // throw-away buffer
      
      // invalidate complete control to make repainting ready
      invalidateRect(this.hWND, 0, true)

      // Start the paint run
      hDC = beginpaint(this.hWND, xPaint)

      // Select bitmap
      cBMPName = this.parent.XPColour + "_TABPANEEDGE_BMP"

      // Top left corner
      this.PaintBitmap(cBMPName, hDC, 0, 0, 2, 2, 0, 0)
      // Top right corner
      this.PaintBitmap(cBMPName, hDC, tw-4, 0, 4, 2, -4, 0)   
      // Bottom left corner
      this.PaintBitmap(cBMPName, hDC, 0, th-4, 2, 4, 0, -4)
      // Bottom left corner
      this.PaintBitmap(cBMPName, hDC, tw-4, th-4, 4, 4, -4, -4)
      // Top edge
		this.PaintBitmapStretch(cBMPName, hDC, 2, 0, tw-6, 2, 2, 0, 10, 2)
      // Left edge
      this.PaintBitmapStretch(cBMPName, hDC, 0, 2, 2, th-6, 0, 2, 2, 10)
      // Right edge
		this.PaintBitmapStretch(cBMPName, hDC, tw-4, 2, 4, th-6, -4, 2, 4, 10)
      // Bottom edge
		this.PaintBitmapStretch(cBMPName, hDC, 2, th-4, tw-6, 4, 2, -4, 10, 4)

      // Pane background
      if this.parent.CurrentColour == "FFFFFF"
         // Draw theme background bitmap
         cBMPName = this.parent.XPColour + "_TABBACKGROUND_BMP"
         this.PaintBitmapStretch(cBMPName, hDC, 2, 2, tw-6, th-6, 0, 0, 10, iif(th<600, th-6, 600))
      else
         // Draw Background Rectangle
         tpCCol = (this.parent.CurrentColour)
         hDefPen = selectobject(hDC, createpen(0, 1, htoi(tpCCol)))
         hDefBrush = selectobject(hDC, createsolidbrush(htoi(tpCCol)))
         Rectangle(hDC, 2, 2, (tw-4), (th-4))
         deleteobject(selectobject(hDC, hDefPen))
         deleteobject(selectobject(hDC, hDefBrush))
      endif

      // Draw Top Edge Break
      cBMPName = this.parent.XPColour + "_TABITEM_BMP"
      if this.parent.aTabRefs[1].Selected == true
         // First Tab
         nTabStart = this.parent.CurrentTabLeft + 1  
         nWidthDest = this.parent.CurrentTabWidth 
      elseif this.parent.aTabRefs[this.parent.TabCount].Selected == true
         // Last Tab
         nTabStart = this.parent.CurrentTabLeft - 1 
         nWidthDest = this.parent.CurrentTabWidth 
      else
         // Flanked Tab
         nTabStart = this.parent.CurrentTabLeft - 1 
         nWidthDest = this.parent.CurrentTabWidth + 2
      endif
      this.PaintBitmapStretch(cBMPName, hDC, nTabStart, 0, nWidthDest, 1, 0, 17+36, 18, 1)
 
      // Finish Painting
      EndPaint(this.hWND,xPaint) 
   
   return

   // -------------------------------------

   function CreateTab(cTabText, cTabColour, nTabTargetPage, nTabWidth, cTabIconName)

      private sMacString

      // Update control properties
      this.TabCount = this.TabCount + 1
      if this.TabCount = 1
         this.CurrentColour = substr(cTabColour,3)
         this.CurrentTabWidth = (nTabWidth)
      endif

      // Create the tab
      sMacString = "this.PAINTTAB" + this.TabCount + "= new rmXPPaintTab(this)"
      &sMacString.
      sMacString = "this.PAINTTAB" + this.TabCount
      oTab = &sMacString.
      oTab.Left = this.NextTabLeft
      oTab.TabText = cTabText
      oTab.TabColour = substr(cTabColour,3)
      oTab.TabTargetPage = nTabTargetPage
      oTab.Width = nTabWidth
      oTab.TabNumber = this.TabCount
      if not empty(argvector(5))
         oTab.IconName = cTabIconName
      else
         oTab.IconName = ""
      endif

      // Add to Array of tab references
      this.aTabRefs.add(oTab)

      // Load Bitmap Resource dll
      oTab.LoadDLL("rmXPLook.dll")

      // Update control properties
      this.NextTabLeft += nTabWidth

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

/*

	Classname:  rmXPPaintTab

   Version:    1.05.0618
    
   Date:       May 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmXPTabBookFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

               It is the duty of the user to determine it's 
               suitability for use. In no circumstance will the
               author accept any responsibility for any 
               consequence arising from it's use.
	
   
   Description:

      A custom paintbox used for the tabs of the rmXPPaintTab class.


   Revisions:

      None.


   Methods:

      None.

   Useage:

      Used by the rmXPTabBook class.


   Dependancies:

      None.
      

   Acknowledgements:

      None.  
         
*/


class rmXPPaintTab(f) Of rmPaintBox(f) Custom
   with (this)
      onPaint = class::TabonPaint
      onLeftMouseDown = class::TabOnLeftMouseDown
      top = 0
      height = 20
      width = 80
      left = 0
      ColorNormal = "N/" + this.parent.parent.ColorNormal
      pageno = 0
   endwith


   // Custom properties
   this.TabText = "TabText"
   this.TabColour = "C0C0C0"
   this.TabTargetPage = 0

   this.TabNumber = 0
   this.Selected = false
   this.IsMouseOver = false

   // -------------------------------------   

   Function TabOnPaint()

      local hDC, hDefPen, hDefBrush, xPaint, tw, cBMPName, nIndent, nOffset, sColour

      tw = this.width
      th = this.height
       
      // invalidate complete control to make repainting ready
      xPaint  = space(32) //throw-away buffer
      invalidateRect(this.hWND, 0, true)

      // Start the paint run
      hDC = beginpaint(this.hWND, xPaint)

      // Select bitmap
      cBMPName = this.parent.XPColour + "_TABITEM_BMP"

      // Get Indent - First tab left edge needs indented two pixels when not selected
      if this.TabNumber == 1
         nIndent = 2
      else
         nIndent = 0
      endif

      // Paint Background 
      sColour = substr("" + this.parent.parent.ColorNormal, 3) 
      hDefPen = selectobject(hDC, createpen(0, 1, htoi(sColour)))
      hDefBrush = selectobject(hDC, createsolidbrush(htoi(sColour)))
      Rectangle(hDC, 0, 0, tw, th) 
      Rectangle(hDC, 0, 0, tw, th)
      deleteobject(selectobject(hDC, hDefPen))
      deleteobject(selectobject(hDC, hDefBrush))

      // Paint tab according to state
      do case

         case this.parent.Disabled == true

            /*
            Since this is a fake notebook, to disable the notebook would involve
            going through all the form controls to find those with a pageNo
            which matched the notebook tab target page and disable those as well.

            Leave this capability out meantime.
            */

         case this.Selected == true

            nOffset = 36

            // Left edge
            if this.TabNumber == 1
               this.PaintBitmap(cBMPName, hDC, 0, 0, 3, 10, 0, 0 + nOffset)
               this.PaintBitmap(cBMPName, hDC, 0, 10, 3, 11, 0, 7 + nOffset)
            else
               this.PaintBitmap(cBMPName, hDC, 0, 0, 1, 10, 2, 0 + nOffset)
               this.PaintBitmap(cBMPName, hDC, 0, 10, 1, 10, 2, 8 + nOffset)
            endif

            // Right edge
            if this.TabNumber == this.parent.TabCount
               this.PaintBitmap(cBMPName, hDC, tw-4, 0, 4, 10, 14, 0 + nOffset)
               this.PaintBitmap(cBMPName, hDC, tw-4, 10, 4, 11, 14, 7 + nOffset)
            else
               this.PaintBitmap(cBMPName, hDC, tw-1, 0, 1, 10, 15, 0 + nOffset)
               this.PaintBitmap(cBMPName, hDC, tw-1, 10, 1, 11, 15, 8 + nOffset)
            endif

            // Tab Body
            if this.TabNumber == 1
               this.PaintBitmapStretch(cBMPName, hDC, 3, 0, tw-4, 20, 4, 0 + nOffset, 11, 18)
            elseif this.TabNumber == this.parent.TabCount
               this.PaintBitmapStretch(cBMPName, hDC, 1, 0, tw-5, 20, 4, 0 + nOffset, 11, 18)
            else
               this.PaintBitmapStretch(cBMPName, hDC, 1, 0, tw-2, 20, 4, 0 + nOffset, 11, 18)
            endif

         otherwise // Normal Tab (hot if required)

            if this.IsMouseOver == true and this.Selected == false 
               // Hot
               nOffset = 18
            else
               // Normal
               nOffset = 0
            endif

            // Left edge
            this.PaintBitmap(cBMPName, hDC, nIndent, 2, 3, 18, 0, 0 + nOffset)
            // Right edge
            this.PaintBitmap(cBMPName, hDC, this.width-3, 2, 4, 18, 15, 0 + nOffset)
            // Tab Body
            this.PaintBitmapStretch(cBMPName, hDC, 3+nIndent, 2, tw-6-nIndent, 18, 4, 0 + nOffset, 11, 18)
            // Left edge overpaint
            if this.TabNumber > 1 and this.parent.aTabRefs[this.TabNumber-1].Selected = true
               nOffset = 36
               this.PaintBitmap(cBMPName, hDC, 0, 0, 2, 10, 16, 0 + nOffset)
               this.PaintBitmap(cBMPName, hDC, 0, 10, 2, 11, 16, 7 + nOffset)
            endif
            // Right edge overpaint
            if this.TabNumber < this.parent.TabCount and this.parent.aTabRefs[this.TabNumber+1].Selected = true
               nOffset = 36
               this.PaintBitmap(cBMPName, hDC, tw-2, 0, 2, 10, 0, 0 + nOffset)
               this.PaintBitmap(cBMPName, hDC, tw-2, 10, 2, 11, 0, 7 + nOffset)
            endif

      endcase

      // Draw Icon
      nBMPWidth = 0
      if not this.IconName == ""
         drop = iif(this.Selected == true, 4, 5)
         // Select bitmap
         cBMPName = this.IconName
         nBMPWidth := this.PaintTransparentBitmap(cBMPName, hDC, 5, drop, "800080")
         nBMPWidth := nBMPWidth + 2 
      endif
     
      // Draw text
      drop = iif(this.Selected == true, 4, 5)
      SetBkMode(hdc, 0) // Transparent
      hFont = CreateFont(13,0,0,0,400,0,0,0,0,0,0,4,0,"Tahoma")
      hNewFont = selectobject(hdc, hFont)  
      TextOut(hDC, 8 + nIndent + nBMPWidth, drop, this.TabText, len(this.TabText))

      // Finish Painting
      EndPaint(this.hWND, xPaint) 
   
   return

   // -------------------------------------

   function TabOnLeftMouseDown()

      // Take focus
      this.setFocus() 

      // Release Mouse capture and update tab properties
      ReleaseCapture()  
      this.IsMouseOver := false

      if this.Selected == false

         // Update standard tabs
         for nCount = 1 to this.parent.aTabRefs.size
            this.parent.aTabRefs[nCount].selected := false
         endfor

         // Update selected tab
         this.selected := true

         // Repaint tabs
         for nCount = 1 to this.parent.aTabRefs.size
            this.parent.aTabRefs[nCount].onPaint()
         endfor

         // Paint PageArea
         this.parent.CurrentColour = (this.TabColour)
         this.parent.CurrentTabLeft = (this.left)
         this.parent.CurrentTabWidth = (this.width)
         form.pageno = this.TabTargetPage

         // Changing the form page initiates a repaint
         // but is not 100% reliable - so force
         this.parent.PageArea.onPaint()

      endif

   return

   // -------------------------------------

   function onMouseMove(flags, col, row)

      // Is mouse over logic - for "hot" tab
      if this.selected == false
         if col > this.width or row > this.height or col < 0 or row < 0
            this.IsMouseOver := false
            ReleaseCapture()
            this.onPaint()
         else
            if not this.IsMouseOver == true
               this.IsMouseOver := true
               SetCapture(this.hwnd)
               this.onPaint()
            endif
         endif
      endif

   return

   // -------------------------------------
   

EndClass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

/*

	Classname:  rmXPTabBook_TL

   Version:    1.05.0723
    
   Date:       May 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmXPTabBookFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

               It is the duty of the user to determine it's 
               suitability for use. In no circumstance will the
               author accept any responsibility for any 
               consequence arising from it's use.
	
   
   Description:

      A custom TextLabel control for use with the rmXPTabBook control.

      Since the background for the rmXPTabBook is a gradient bitmap, controls
      need to have their background colour matched to their vertical position 
      in the rmXPTabBook control. This control will automatically find its
      associated rmXPTabBook control, and set its background colour accordingly.

      This does however create an overhead, and if this is a problem when using
      a large numberof text controls it may be better to hard code the
      background colours accordingly.


   Revisions:

      2005/07/23

         This custom control can now be used where a tab page
         background colour has been defined. The background colour of this
         control will be automatically matched to the rmXPTabBook page colour.


   Methods:

      None.

   Useage:

      See rmXPTabBook.wfm for sample code.


   Dependancies:

      None.
      

   Acknowledgements:

      None.  
         
*/


Class rmXPTabBook_TL(oF) of TEXTLABEL(oF) Custom 

   this.onOpen = CLASS::SetBackgroundColour

   // -------------------------------------

   function SetBackgroundColour()

      local nhBackCol, oRef, nVertPos

      nVertPos = ""

      // Find rmXPTabBook controls
      for nCount = 1 to form.elements.size
         if form.elements[nCount].classname == "RMXPTABBOOK"
            oRef = form.elements[nCount]
            if this.top > oRef.top and this.top < oRef.top + oRef.height
               if this.left > oRef.left and this.left < oRef.left + oRef.width 
                  // Associated rmXPTabBook found
                  nVertPos = this.top - oRef.top
                  exit
               endif
            endif
         endif
      next

      // Trap for no associated rmXPTabBook
      if nVertPos == ""
         msgbox(this.name + " is not positioned over a rmXPTabBook control", "Error", 0+16)
         return
      endif

      // Find which tab displays the form page this control is on
      for nCount = 1 to oRef.aTabRefs.size
         if oRef.aTabRefs[nCount].TabTargetPage == this.pageNo
            this.TabColour = oRef.aTabRefs[nCount].TabColour
         endif
      endfor // next

      // Set Background colour 
      if this.TabColour == "FFFFFF"

         // XP gradient match
         // Set background colour to match position
         do case
            case nVertPos <= 50
               nhBackCol = "0xFDFBFB"
            case nVertPos <= 100
               nhBackCol = "0xFBFBFA"
            case nVertPos <= 150
               nhBackCol = "0xF7F8F8"
            case nVertPos <= 200
               nhBackCol = "0xF4F7F7"
            case nVertPos <= 250
               nhBackCol = "0xF1F4F5"
            case nVertPos <= 300
               nhBackCol = "0xEFF3F5"
            otherwise
               nhBackCol = "0xEEF3F4"
         endcase
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + nhBackCol

      else

         // User defined TabBook page colour
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + "0x" + this.TabColour

      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

/*

	Classname:  rmXPTabBook_TXT

   Version:    1.05.0723
    
   Date:       May 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmXPTabBookFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

               It is the duty of the user to determine it's 
               suitability for use. In no circumstance will the
               author accept any responsibility for any 
               consequence arising from it's use.
	
   
   Description:

      A custom Text control for use with the rmXPTabBook control.

      Since the background for the rmXPTabBook is a gradient bitmap, controls
      need to have their background colour matched to their vertical position 
      in the rmXPTabBook control. This control will automatically find its
      associated rmXPTabBook control, and set its background colour accordingly.

      This does however create an overhead, and if this is a problem when using
      a large numberof text controls it may be better to hard code the
      background colours accordingly.


   Revisions:

      2005/07/23

         This custom control can now be used where a tab page
         background colour has been defined. The background colour of this
         control will be automatically matched to the rmXPTabBook page colour.


   Methods:

      None.

   Useage:

      See rmXPTabBook.wfm for sample code.


   Dependancies:

      None.
      

   Acknowledgements:

      None.  
         
*/


Class rmXPTabBook_TXT(oF) of TEXT(oF) Custom 

   this.onOpen = CLASS::SetBackgroundColour

   // -------------------------------------

   function SetBackgroundColour()

      local nhBackCol, oRef, nVertPos

      nVertPos = ""

      // Find rmXPTabBook controls
      for nCount = 1 to form.elements.size
         if form.elements[nCount].classname == "RMXPTABBOOK"
            oRef = form.elements[nCount]
            if this.top > oRef.top and this.top < oRef.top + oRef.height
               if this.left > oRef.left and this.left < oRef.left + oRef.width 
                  // Associated rmXPTabBook found
                  nVertPos = this.top - oRef.top
                  exit
               endif
            endif
         endif
      next

      // Trap for no associated rmXPTabBook
      if nVertPos == ""
         msgbox(this.name + " is not positioned over a rmXPTabBook control", "Error", 0+16)
         return
      endif

      // Find which tab displays the form page this control is on
      for nCount = 1 to oRef.aTabRefs.size
         if oRef.aTabRefs[nCount].TabTargetPage == this.pageNo
            this.TabColour = oRef.aTabRefs[nCount].TabColour
         endif
      endfor // next

      // Set Background colour 
      if this.TabColour == "FFFFFF"

         // XP gradient match
         // Set background colour to match position
         do case
            case nVertPos <= 50
               nhBackCol = "0xFDFBFB"
            case nVertPos <= 100
               nhBackCol = "0xFBFBFA"
            case nVertPos <= 150
               nhBackCol = "0xF7F8F8"
            case nVertPos <= 200
               nhBackCol = "0xF4F7F7"
            case nVertPos <= 250
               nhBackCol = "0xF1F4F5"
            case nVertPos <= 300
               nhBackCol = "0xEFF3F5"
            otherwise
               nhBackCol = "0xEEF3F4"
         endcase
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + nhBackCol

      else

         // User defined TabBook page colour
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + "0x" + this.TabColour

      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

/*

	Classname:  rmXPTabBook_CHBOX

   Version:    1.05.0723
    
   Date:       May 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmXPTabBookFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

               It is the duty of the user to determine it's 
               suitability for use. In no circumstance will the
               author accept any responsibility for any 
               consequence arising from it's use.
	
   
   Description:

      A custom CheckBox control for use with the rmXPTabBook control.

      Since the background for the rmXPTabBook is a gradient bitmap, controls
      need to have their background colour matched to their vertical position 
      in the rmXPTabBook control. This control will automatically find its
      associated rmXPTabBook control, and set its background colour accordingly.

      This does however create an overhead, and if this is a problem when using
      a large numberof text controls it may be better to hard code the
      background colours accordingly.


   Revisions:

      2005/07/23

         This custom control can now be used where a tab page
         background colour has been defined. The background colour of this
         control will be automatically matched to the rmXPTabBook page colour.


   Methods:

      None.

   Useage:

      See rmXPTabBook.wfm for sample code.


   Dependancies:

      None.
      

   Acknowledgements:

      None.  
         
*/


Class rmXPTabBook_CHBOX(oF) of CHECKBOX(oF) Custom 

   this.onOpen = CLASS::SetBackgroundColour

   // -------------------------------------

   function SetBackgroundColour()

      local nhBackCol, oRef, nVertPos

      nVertPos = ""

      // Find rmXPTabBook controls
      for nCount = 1 to form.elements.size
         if form.elements[nCount].classname == "RMXPTABBOOK"
            oRef = form.elements[nCount]
            if this.top > oRef.top and this.top < oRef.top + oRef.height
               if this.left > oRef.left and this.left < oRef.left + oRef.width 
                  // Associated rmXPTabBook found
                  nVertPos = this.top - oRef.top
                  exit
               endif
            endif
         endif
      next

      // Trap for no associated rmXPTabBook
      if nVertPos == ""
         msgbox(this.name + " is not positioned over a rmXPTabBook control", "Error", 0+16)
         return
      endif

      // Find which tab displays the form page this control is on
      for nCount = 1 to oRef.aTabRefs.size
         if oRef.aTabRefs[nCount].TabTargetPage == this.pageNo
            this.TabColour = oRef.aTabRefs[nCount].TabColour
         endif
      endfor // next

      // Set Background colour 
      if this.TabColour == "FFFFFF"

         // XP gradient match
         // Set background colour to match position
         do case
            case nVertPos <= 50
               nhBackCol = "0xFDFBFB"
            case nVertPos <= 100
               nhBackCol = "0xFBFBFA"
            case nVertPos <= 150
               nhBackCol = "0xF7F8F8"
            case nVertPos <= 200
               nhBackCol = "0xF4F7F7"
            case nVertPos <= 250
               nhBackCol = "0xF1F4F5"
            case nVertPos <= 300
               nhBackCol = "0xEFF3F5"
            otherwise
               nhBackCol = "0xEEF3F4"
         endcase
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + nhBackCol

      else

         // User defined TabBook page colour
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + "0x" + this.TabColour

      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

/*

	Classname:  rmXPTabBook_SL

   Version:    1.05.0723
    
   Date:       May 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmXPTabBookFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

               It is the duty of the user to determine it's 
               suitability for use. In no circumstance will the
               author accept any responsibility for any 
               consequence arising from it's use.
	
   
   Description:

      A custom Slider control for use with the rmXPTabBook control.

      Since the background for the rmXPTabBook is a gradient bitmap, controls
      need to have their background colour matched to their vertical position 
      in the rmXPTabBook control. This control will automatically find its
      associated rmXPTabBook control, and set its background colour accordingly.

      This does however create an overhead, and if this is a problem when using
      a large numberof text controls it may be better to hard code the
      background colours accordingly.


   Revisions:

      2005/07/23

         This custom control can now be used where a tab page
         background colour has been defined. The background colour of this
         control will be automatically matched to the rmXPTabBook page colour.


   Methods:

      None.

   Useage:

      See rmXPTabBook.wfm for sample code.


   Dependancies:

      None.
      

   Acknowledgements:

      None.  
         
*/


Class rmXPTabBook_SL(oF) of SLIDER(oF) Custom 

   this.onOpen = CLASS::SetBackgroundColour

   // -------------------------------------

   function SetBackgroundColour()

      local nhBackCol, oRef, nVertPos

      nVertPos = ""

      // Find rmXPTabBook controls
      for nCount = 1 to form.elements.size
         if form.elements[nCount].classname == "RMXPTABBOOK"
            oRef = form.elements[nCount]
            if this.top > oRef.top and this.top < oRef.top + oRef.height
               if this.left > oRef.left and this.left < oRef.left + oRef.width 
                  // Associated rmXPTabBook found
                  nVertPos = this.top - oRef.top
                  exit
               endif
            endif
         endif
      next

      // Trap for no associated rmXPTabBook
      if nVertPos == ""
         msgbox(this.name + " is not positioned over a rmXPTabBook control", "Error", 0+16)
         return
      endif

      // Find which tab displays the form page this control is on
      for nCount = 1 to oRef.aTabRefs.size
         if oRef.aTabRefs[nCount].TabTargetPage == this.pageNo
            this.TabColour = oRef.aTabRefs[nCount].TabColour
         endif
      endfor // next

      // Set Background colour 
      if this.TabColour == "FFFFFF"

         // XP gradient match
         // Set background colour to match position
         do case
            case nVertPos <= 50
               nhBackCol = "0xFDFBFB"
            case nVertPos <= 100
               nhBackCol = "0xFBFBFA"
            case nVertPos <= 150
               nhBackCol = "0xF7F8F8"
            case nVertPos <= 200
               nhBackCol = "0xF4F7F7"
            case nVertPos <= 250
               nhBackCol = "0xF1F4F5"
            case nVertPos <= 300
               nhBackCol = "0xEFF3F5"
            otherwise
               nhBackCol = "0xEEF3F4"
         endcase
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + nhBackCol

      else

         // User defined TabBook page colour
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + "0x" + this.TabColour

      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

/*

	Classname:  rmXPTabBook_RB

   Version:    1.05.0723
    
   Date:       May 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmXPTabBookFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

               It is the duty of the user to determine it's 
               suitability for use. In no circumstance will the
               author accept any responsibility for any 
               consequence arising from it's use.
	
   
   Description:

      A custom RadioButton control for use with the rmXPTabBook control.

      Since the background for the rmXPTabBook is a gradient bitmap, controls
      need to have their background colour matched to their vertical position 
      in the rmXPTabBook control. This control will automatically find its
      associated rmXPTabBook control, and set its background colour accordingly.

      This does however create an overhead, and if this is a problem when using
      a large numberof text controls it may be better to hard code the
      background colours accordingly.


   Revisions:

      2005/07/23

         This custom control can now be used where a tab page
         background colour has been defined. The background colour of this
         control will be automatically matched to the rmXPTabBook page colour.


   Methods:

      None.

   Useage:

      See rmXPTabBook.wfm for sample code.


   Dependancies:

      None.
      

   Acknowledgements:

      None.  
         
*/


Class rmXPTabBook_RB(oF) of RADIOBUTTON(oF) Custom 

   this.onOpen = CLASS::SetBackgroundColour

   // -------------------------------------

   function SetBackgroundColour()

      local nhBackCol, oRef, nVertPos

      nVertPos = ""

      // Find rmXPTabBook controls
      for nCount = 1 to form.elements.size
         if form.elements[nCount].classname == "RMXPTABBOOK"
            oRef = form.elements[nCount]
            if this.top > oRef.top and this.top < oRef.top + oRef.height
               if this.left > oRef.left and this.left < oRef.left + oRef.width 
                  // Associated rmXPTabBook found
                  nVertPos = this.top - oRef.top
                  exit
               endif
            endif
         endif
      next

      // Trap for no associated rmXPTabBook
      if nVertPos == ""
         msgbox(this.name + " is not positioned over a rmXPTabBook control", "Error", 0+16)
         return
      endif

      // Find which tab displays the form page this control is on
      for nCount = 1 to oRef.aTabRefs.size
         if oRef.aTabRefs[nCount].TabTargetPage == this.pageNo
            this.TabColour = oRef.aTabRefs[nCount].TabColour
         endif
      endfor // next

      // Set Background colour 
      if this.TabColour == "FFFFFF"

         // XP gradient match
         // Set background colour to match position
         do case
            case nVertPos <= 50
               nhBackCol = "0xFDFBFB"
            case nVertPos <= 100
               nhBackCol = "0xFBFBFA"
            case nVertPos <= 150
               nhBackCol = "0xF7F8F8"
            case nVertPos <= 200
               nhBackCol = "0xF4F7F7"
            case nVertPos <= 250
               nhBackCol = "0xF1F4F5"
            case nVertPos <= 300
               nhBackCol = "0xEFF3F5"
            otherwise
               nhBackCol = "0xEEF3F4"
         endcase
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + nhBackCol

      else

         // User defined TabBook page colour
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + "0x" + this.TabColour

      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

/*

	Classname:  rmXPTabBook_RECT

   Version:    1.05.0723
    
   Date:       May 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmXPTabBookFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

               It is the duty of the user to determine it's 
               suitability for use. In no circumstance will the
               author accept any responsibility for any 
               consequence arising from it's use.
	
   
   Description:

      A custom Rectangle control for use with the rmXPTabBook control.

      Since the background for the rmXPTabBook is a gradient bitmap, controls
      need to have their background colour matched to their vertical position 
      in the rmXPTabBook control. This control will automatically find its
      associated rmXPTabBook control, and set its background colour accordingly.

      This does however create an overhead, and if this is a problem when using
      a large numberof text controls it may be better to hard code the
      background colours accordingly.


   Revisions:

      2005/07/23

         This custom control can now be used where a tab page
         background colour has been defined. The background colour of this
         control will be automatically matched to the rmXPTabBook page colour.


   Methods:

      None.

   Useage:

      See rmXPTabBook.wfm for sample code.


   Dependancies:

      None.
      

   Acknowledgements:

      None.  
         
*/


Class rmXPTabBook_RECT(oF) of RECTANGLE(oF) Custom 

   this.onOpen = CLASS::SetBackgroundColour

   // -------------------------------------

   function SetBackgroundColour()

      local nhBackCol, oRef, nVertPos

      nVertPos = ""

      // Find rmXPTabBook controls
      for nCount = 1 to form.elements.size
         if form.elements[nCount].classname == "RMXPTABBOOK"
            oRef = form.elements[nCount]
            if this.top > oRef.top and this.top < oRef.top + oRef.height
               if this.left > oRef.left and this.left < oRef.left + oRef.width 
                  // Associated rmXPTabBook found
                  nVertPos = this.top - oRef.top
                  exit
               endif
            endif
         endif
      next

      // Trap for no associated rmXPTabBook
      if nVertPos == ""
         msgbox(this.name + " is not positioned over a rmXPTabBook control", "Error", 0+16)
         return
      endif

      // Find which tab displays the form page this control is on
      for nCount = 1 to oRef.aTabRefs.size
         if oRef.aTabRefs[nCount].TabTargetPage == this.pageNo
            this.TabColour = oRef.aTabRefs[nCount].TabColour
         endif
      endfor // next

      // Set Background colour 
      if this.TabColour == "FFFFFF"

         // XP gradient match
         // Set background colour to match position
         do case
            case nVertPos <= 50
               nhBackCol = "0xFDFBFB"
            case nVertPos <= 100
               nhBackCol = "0xFBFBFA"
            case nVertPos <= 150
               nhBackCol = "0xF7F8F8"
            case nVertPos <= 200
               nhBackCol = "0xF4F7F7"
            case nVertPos <= 250
               nhBackCol = "0xF1F4F5"
            case nVertPos <= 300
               nhBackCol = "0xEFF3F5"
            otherwise
               nhBackCol = "0xEEF3F4"
         endcase
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + nhBackCol

      else

         // User defined TabBook page colour
         sColour = "" + this.colorNormal
         this.colorNormal = substr(sColour, 1, at("/", sColour, 1)) + "0x" + this.TabColour

      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// eof
